/*
 * IconFormat.java September 2002
 *
 * Copyright (C) 2002, Niall Gallagher <niallg@email.com>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free 
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

package mamba.format;

import java.util.MissingResourceException;
import java.util.ResourceBundle;
import simple.http.serve.FormatException;
import simple.http.serve.Format;
import simple.http.serve.Context;
import simple.http.serve.Report;
import java.text.SimpleDateFormat;
import java.text.DateFormat;
import java.util.Locale;
import java.util.Date;
import java.util.Arrays;
import java.io.File;

/** 
 * A <code>IconFormat</code> implementation for providing output 
 * to a <code>ResourceEngine</code> that is formatted similar to 
 * the Apache web server. 
 *
 * @author Niall Gallagher
 */
public class IconFormat implements Format {

   protected static ResourceBundle icons;   
   
   static {      
      try { 
         icons = ResourceBundle.getBundle(
         "mamba.format.Icons", Locale.ENGLISH);
      }catch(MissingResourceException e){
         e.printStackTrace();
      }   
   }   

   public byte[] getContents(Context context, String target)
      throws FormatException{
      if(!context.getFile(target).isDirectory()){
         throw new FormatException("No format available");
      }         
      String fixed = target;
      
      if(fixed.indexOf('?')>0){ /* remove query*/
         fixed = fixed.substring(0,
            fixed.indexOf('/')+1);
      }
      if(fixed.lastIndexOf(';')>0){ /* remove params*/
         fixed = fixed.substring(0,
            fixed.indexOf(';')+1);
      }
      return getContents(context, target, 
         !fixed.endsWith("/"));
   }
   
   public byte[] getContents(Context context, String target, Object[] data)
      throws FormatException {
      return getContents(context, target);
   }

   private byte[] getContents(Context context, String target, boolean isRelative){
      DateFormat format = new SimpleDateFormat("dd-MMM-yyyy HH:mm");
      String prefix = isRelative ? (context.getName(target)+"/"):"";
      File dir = context.getFile(target);
      String[] names = dir.list();
      Arrays.sort(names);      
      
      String text = 
      "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">"+
      "<HTML>\n<HEAD>\n\t"+
      "<TITLE>Index of "+target+"</TITLE>\n"+
      "</HEAD>\n<BODY>\n" +
      "<H1>Index of "+target+"</H1>\n" +
      "<TABLE>\n<TR>\n" +
      "<TD><IMG SRC=/icons/blank.gif></TD>\n"+
      "<TD WIDTH=200><B>Name</B></TD>\n"+
      "<TD WIDTH=140><B>Last Modified</B></TD>\n"+
      "<TD WIDTH=60><B>Size</B></TD>\n"+
      "<TD><B>Description</B></TD>\n"+
      "</TR>\n</TABLE>\n<HR>"+
      "<TABLE>\n<TR>"+
      "<TD><IMG SRC=/icons/back.gif></TD>\n"+
      "<TD WIDTH=200><A HREF=../>Parent Directory</A></TD>\n"+
      "<TD WIDTH=140>"+format.format(new Date())+"</TD>\n"+
      "<TD WIDTH=60>-</TD>\n<TD>text/html</TD></TR>\n"; 

      for(int i = 0; i < names.length; i++) {
         File file = new File(dir, names[i]);
         String icon, path, name, size, type;
         Date date = null;
         if(file.isDirectory()){
            icon = "/icons/folder.gif";
            date = new Date(file.lastModified());
            size = "-";
            name = names[i] + "/";
            path = prefix + name;           
            type = "text/html";
         }else {
            long kilo = file.length() / 1024L;
            name = names[i];
            path = prefix + name;
            date = new Date(file.lastModified());
            size = ""+Math.max(kilo, 1L)+"k";      
            type = context.getMimeType("/"+name);      
            try{
               icon = icons.getString(type);
            }catch(Exception e){
               icon = "/icons/unknown.gif";
            }
         }     
         text += "<TR><TD><IMG SRC=\""+icon+"\"></TD>\n"+
         "<TD WIDTH=200><A HREF=\""+path+"\">"+name+"</A></TD>\n"+
         "<TD WIDTH=140>\t"+format.format(date)+"</TD>\n"+
         "<TD WIDTH=60>\t"+size+"</TD>\n<TD>"+type+"</TD>\n</TR>\n";
      }    
      return getBytes(text+"</TABLE><HR></BODY></HTML>");
   }

   public byte[] getMessage(Context context, String target, Report report){
      return  getBytes("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n"+
         "<HTML><HEAD><TITLE>"+report.getCode() + " "+report.getText()+"</TITLE>\n"+
         "</HEAD><BODY><H1>"+report.getText()+"</H1>"+
         "<PRE>The requested URL <i>"+target+"</i></PRE>"+
         "<HR><PRE>"+report.getCause()+"</PRE></BODY></HTML>");
   }

   private byte[] getBytes(String text){      
      try {
         return text.getBytes("utf-8");
      }catch(Exception e) {
         return new byte[0];
      }
   }

   public String getMimeType() {
      return "text/html; charset=utf-8";
   }
}
