/*
 * ServerHandler.java January 2003
 *
 * Copyright (C) 2003, Niall Gallagher <niallg@email.com>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free 
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

package mamba.serve;

import simple.http.ProtocolHandler;
import simple.http.Response;
import simple.http.Request;
import java.net.InetAddress;
import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.File;

/**
 * This provides a filter for the <code>Serve</code> server so
 * that service implementations do not have to worry about HTTP.
 *
 * @author Niall Gallagher
 */
public class ServerHandler implements ProtocolHandler {
   
   private ProtocolHandler handler;

   private PrintWriter log;

   public ServerHandler(ProtocolHandler handler, File path) throws Exception {
      this.log = new PrintWriter(new FileWriter(path, true), true);
      this.handler = handler;
   }

   public void handle(Request req, Response resp) {
      resp.set("Server", "Mamba/1.0 (Simple/2.3.3)");
      resp.setDate("Date", System.currentTimeMillis());
   
      /*
       * This line is used to log each hit that occurs
       * on the server. The information logged includes
       * the user agent and the refferal address.
       */
      log.println(resp.getValue("Date")+ "," + req.getMethod() +","+
         req.getURI()+","+req.getValue("Referer")+","+
        req.getInetAddress().getHostAddress()+","+
         req.getValue("User-Agent"));

      /*
       * Hand the request over to the real server so
       * that targeted service can be executed.
       */
      handler.handle(req, resp);
   }
}   
