/*
 * IndexService.java January 2003
 *
 * Copyright (C) 2003, Niall Gallagher <niallg@email.com>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free 
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

package mamba.serve.service;

import simple.http.load.BasicService;
import simple.http.load.LoaderEngine;
import simple.http.serve.FileEngine;
import simple.util.parse.URIParser;
import simple.http.serve.Context;
import simple.http.serve.Resource;
import simple.http.Response;
import simple.http.Request;
import simple.util.net.URI;

/**
 * This provides a simple forwarding mechanism that enables the
 * referenced URI is appended with <code>index.htm(l)</code>. 
 * 
 * @author Niall Gallagher
 */
public class IndexService extends BasicService {

   private FileEngine engine;

   public IndexService(Context context) {
      super(context);
   }

   public void prepare(LoaderEngine loader, Object data){
      engine = new FileEngine(context);
   }

   public void process(Request req, Response resp) {      
      String path = getIndexPath(req.getURI());      
      
      engine.lookup(path).handle(req, resp);
   }

   private String getIndexPath(String request) {
      URI target = new URIParser(request);
      String path = target.getPath().getPath();

      if(isValid(path + "index.html")){
         target.setPath(path + "index.html");
      }else if(isValid(path + "index.htm")){
         target.setPath(path + "index.htm");
      }
      return target.toString();
   }     

   private boolean isValid(String target) {
      return context.getFile(target).exists();
   }   
}

